;================================================
;ACR_OUT
; Program writes a section of memory out to the 
; 88-ACR Cassette Interface. This is useful for 
; making "virgin" copies of old software cassettes.
; Program returns to monitor when finished or if
; user presses Ctrl-X on keyboard.
; John Garza Jun 2010 Conroe
;================================================

ORIGIN	EQU		3E00h	; To give ample space for data
						; Remember 0000-00FF used by monitor		

		ORG		ORIGIN			

;------------------
; *** SET I/O CARD HERE ***

SIOA	EQU		1	;-+
SIO2	EQU		0	;-+-- only one of these can be "1"
SIO2P	EQU		0	; SIO2 PARAMS 01=7E2 15h=8N1

;------------------
; *** SET MONITOR PRESENCE HERE ***

MNTR	EQU		1

;------------------
;RS-232 EQUATES for SIO-A (COM 2502 chip)
IF SIOA

	TTS     equ     00h     ;MITS SIO-A channel A status port
	TTI     equ     01h     ;MITS SIO-A data input port (yes input=output)
	TTO     equ     01h     ;MITS SIO-A data output port
	TTYDA   equ     20h     ;tty data available (ready to receive?) for MITS SIO-A
	TTYTR   equ     02h     ;tty terminal ready (ready to transmit?) for MITS SIO-A

ENDIF
;------------------
;RS-232 EQUATES for 2SIO (6850 chip)
IF SIO2

	TTS     equ     10h     ;MITS 2SIO status/control port
	TTI     equ     11h     ;MITS 2SIO data input port (yes input=output)
	TTO     equ     11h     ;MITS 2SIO data output port
	TTYDA   equ     01h     ;tty data available (ready to receive?) for MITS 2SIO
	TTYTR   equ     02h     ;tty terminal ready (ready to transmit?) for MITS 2SIO

ENDIF

;-------------------------------------------------
ADAT	EQU		7			; ACR DATA channel
ASTA	EQU		6			; ACR STATUS channel
TB		EQU		0			; ACR Test Byte
WRMST	EQU		0E440h		; Altair Monitor 8.1 warmstart address
STACK	EQU		ORIGIN-16	; Stack loaction, grows downwards
;-------------------------------------------------
IBUFP   EQU     STACK+1     ; Buffer Pointer 2BYTES
IBUFC   EQU     IBUFP+2     ; Buffer Count 1BYTE
IBUFF   EQU     IBUFP+3     ; Input Buffer
IBL		EQU		8			; Input Buffer Length 
SL		EQU		STACK+12	; Start Address LO byte
SH		EQU		STACK+13	; Start Address HI byte
EL		EQU		STACK+14	; End Address LO byte
EH		EQU		STACK+15	; End Address HI byte 
;-------------------------------------------------
NUL		EQU		0		; NULL (ZERO)
CR		EQU		0DH		; Carriage return
LF		EQU		0AH		; Line feed
CTRH    EQU     8       ; Ctl H Backspace
DEL     EQU     127     ; Delete char
CTRX    EQU     24      ; Ctrl X Cancel
;=================================================
; MAIN PROGRAM

 
;-------------------------------------------------
; INITIALIZATION
 
    LXI     SP,STACK    ; Initialize stack for welcome msg
    
; INITIALIZE 6850 CHIP
IF SIO2

	MVI		A,03h
	OUT		TTS		; 6850 MASTER RESET
	
	MVI		A,SIO2P	; SET PARAMS
	OUT		TTS
	
ENDIF    
    
        
    CALL    ilprt       ; Display welcome message
    db      lf,cr,'Altair 88-ACR Cassette Write Program',cr,lf,00h
        
;-------------------------------------------------
; GET USER INPUTS
	
	CALL	ilprt
	db		'Start Address: ',00h
	CALL	inpl2	; getline without '>' prompt char
	CALL	readhl
	MOV		A,L
	STA		SL
	MOV		A,H
	STA		SH
	
	CALL	ilprt
	db		'End   Address: ',00h
	CALL	inpl2
	CALL	readhl
	MOV		A,L
	STA		EL
	MOV		A,H
	STA		EH

	CALL	ilprt
	db		'Working...(Ctrl-X exits) ',00h


;-------------------------------------------------
; ACR WRITE


		LHLD	EL		; Load end address in BC
		MOV		B,H		;
		MOV		C,L		;
		LHLD	SL		; Load start address in HL 

		;MVI		A,TB	; Move test byte into A
		;OUT		ADAT	; Send test byte to ACR
		
LOOP:	IN		ASTA	; Get ACR status 
		RLC				; Rotate A left to push D7 into carry bit
		JC		LOOP	; Jump to LOOP if carry true
		MOV		A,M		; Move contents of memory specified by HL to A
		OUT		ADAT	; Send byte in A to ACR
		MOV		A,L		; Move contents of L to A
		CMP		C		; Compare A with C
		JNZ		NXT		; Jump if nonzero to NXT
		MOV		A,H		; Move contents of H to A
		CMP		B		; Compare A with B
		JNZ		NXT		; Jump if nonzero to NXT
		JMP		FIN		; Jump (if H=B and L=C) to FINI
		
NXT:	INX		H		; Increment register pair HL

        IN      TTS             ;CHECK FOR USER INPUT CTRL-X
        ANI     TTYDA           ;
        JZ      LOOP            ;   (NO KEY WAS HIT)
        IN      TTI             ;   (KEY WAS HIT, SO GET IT)
        ANI     DEL             ;
        CPI     CTRX            ;
        JZ      exit           ;   (BACK TO MONITOR IF CTRL-X)
        
		JMP		LOOP	; Jump to LOOP
		
FIN:	CALL	ilprt
		db		'DONE',00h
		JMP		exit	; Jump back to monitor warmstart address

;=================================================
; SUBROUTINES

;Inline Print -----------------------------------
;
;THE CALL TO ILPRT IS FOLLOWED BY A MESSAGE,
;BINARY 0 AS THE END.
;

ilprt:  XTHL            ;SAVE HL, GET HL=MSG
ilplp:  MOV     A,M     ;GET CHAR
		ORA		A		;END OF MSG?
        JZ      ilpret  ;..YES, RETURN
        CALL    outt    ;TYPE THE MSG
		INX		H		;TO NEXT CHAR
        JMP     ilplp   ;LOOP
ilpret: XTHL            ;RESTORE HL
		RET				;PAST MSG


;------------------
;Input line from console and store in buffer
;Ctl-X cancels line, ,BKSP erases last char, CR enters line

inpl2:  LXI     H,IBUFF         ;Input buffer addr
        SHLD    IBUFP
        MVI     C,0             ;Init count to zero

inpli:  CALL    intt            ;Get char from console

        CPI     ' '             ;Control char?
        JC      inplc           ;Yes

        CPI     DEL             ;Delete char?
        JZ      inplb           ;Yes

        CPI     'Z'+1           ;Upper case?
        JC      inpl3           ;Yes
        ANI     5Fh             ;No - so make upper case


inpl3:  MOV     M,A             ;Into buffer
        MVI     A,IBL           ;Buffer size
        CMP     C               ;Full?
        JZ      inpli           ;Yes, loop
        MOV     A,M             ;Get char from buffer
        INX     H               ;Incr pointer
        INR     C               ; and count
inple:  CALL    OUTT            ;Show char
        JMP     inpli           ;Next char

        ;Process control chars

inplc:  CPI     CTRH            ;Ctl H ?
        JZ      inplb           ;Yes
        CPI     CR              ;Return?
        JNZ     inpli           ;No, ignore

        ;End of input line

        MOV     A,C             ;Count
        STA     IBUFC           ;Save it

        ;CR LF routine
crlf:
        MVI     A,CR
        CALL    outt            ;Send CR
        MVI     A,LF
        JMP     outt            ;Send LF

        ;Delete prior char, if any

inplb:  MOV     A,C             ;Char count
        ORA     A               ;Zero?
        JZ      inpli           ;Yes
        DCX     H               ;Back pointer
        DCR     C               ; and count

        MVI     A,CTRH
        JMP     inple           ;Send ctrl H

;--------------------
;input HL from console

readhl: PUSH    D
        PUSH    B       ;Save regs
        LXI     H,0     ;Clear
rdhl2:  CALL    getch   ;Get char
        JC      rdhl5   ;Line end
        CALL    nib     ;To binary
        JC      rdhl4   ;Not hex
        DAD     H       ;Times 2
        DAD     H       ;Times 4
        DAD     H       ;Times 8
        DAD     H       ;Times 16
        ORA     L       ;Add new char
        MOV     L,A
        JMP     rdhl2   ;Next

        ;check for blank at end

rdhl4:  CPI     (' '-'0') AND 0FFH
        JNZ     error   ;N0
rdhl5:  POP     B
        POP     D       ;Restore
        RET

        ;convert ascii chars to binary

nib:    SUI     '0'     ;Ascii bias
        RC              ;<0
        CPI     'F'-'0'+1
        CMC             ;Invert
        RC              ;Error, >F
        CPI     10
        CMC             ;Invert
        RNC             ;Number 0-9
        SUI     'A'-'9'-1
        CPI     10      ;Skip : to
        RET             ;Letter A-F

        ;print ? on improper input

error:  MVI     A,'?'
        CALL    outt
        JMP     exit   ;Try again


;------------------
;Get Character from Console Buffer
; Set Carry Bit if empty

getch:  PUSH    H               ;Save Regs
        LHLD    IBUFP           ;Get Pointer
        LDA     IBUFC           ; and Count
        SUI     1               ;Decr with carry
        JC      getc4           ;No more char
        STA     IBUFC           ;Save new count
        MOV     A,M             ;Get character
        INX     H               ;Incr pointer
        SHLD    IBUFP           ; and save
getc4:  POP     H               ;Restore Regs
        RET
   
;------------------
;Console Input

intt: 
        CALL    instat          ;Check status
        IN      TTI             ;Get byte
        ANI     DEL
        CPI     CTRX            ;Abort?
        JZ      exit           ;
        RET

;------------------
;Console Output

outt:   
        PUSH    PSW
        CALL    outstat         
        POP     PSW
        OUT     TTO             ;Send Data
        RET


;------------------
;Check Console Input Status

instat:
        IN      TTS
        ANI     TTYDA
        JZ      instat
        RET

;------------------
;Check Console Output Status

outstat:
        IN      TTS             ;CHECK FOR USER INPUT CTRL-X
        ANI     TTYDA           ;
        JZ      out2            ;
        IN      TTI             ;
        ANI     DEL             ;
        CPI     CTRX            ;
        JZ      exit           ;

out2:   IN      TTS             ;Check Console Output Statuas
        ANI     TTYTR           ;
        JZ      outstat         ;
        RET
;-------------------
; Exit Routine

exit:

IF MNTR
		JMP		WRMST
ELSE
		HLT
ENDIF


;==========
	END
;==========
